/**
 * Copyright (c) Members of the EGEE Collaboration. 2004-2010. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 *
 *  Authors:
 *  2009-
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     Mischa Sall\'e <msalle@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *     <grid-mw-security@nikhef.nl> 
 *
 *  2007-2009
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *
 *  2003-2007
 *     Martijn Steenbakkers <martijn@nikhef.nl>
 *     Gerben Venekamp <venekamp@nikhef.nl>
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* For the gss_cred_id_t struct (input) */
#include <gssapi.h>

/******************************************************************************
Function:       gss_cred_to_dn() (copied from GLOBUS gatekeeper.c)
Description:    Get the globusid from gssapi
Parameters:
                globus_cred: globus credential
Returns:        globusid string (which could be freeed)
******************************************************************************/
char * gss_cred_to_dn(
        gss_cred_id_t globus_cred
)
{
    char*                         globusid = NULL;
    char*                         globusid_tmp = NULL;
    gss_name_t                    globus_name = GSS_C_NO_NAME;
    gss_buffer_desc               globus_buffer_desc = GSS_C_EMPTY_BUFFER;
    gss_buffer_t                  globus_buffer = &globus_buffer_desc;
    OM_uint32                     major_status = 0;
    OM_uint32                     minor_status = 0;
    OM_uint32                     minor_status2 = 0;

    if ((major_status = gss_inquire_cred(&minor_status,
                                         globus_cred,
                                         &globus_name,
                                         NULL, NULL, NULL)) == GSS_S_COMPLETE)
    {
        major_status = gss_display_name(&minor_status,
                                        globus_name, globus_buffer, NULL);
        gss_release_name(&minor_status2, &globus_name);
    }
    /*
     * The gssapi_cleartext does not implement gss_inquire_cred,
     * so fall back to using environment variable.
     */
    if (major_status == GSS_S_COMPLETE)
    {
        globusid = globus_buffer_desc.value;
    }
    else
    {
        globusid = getenv("GLOBUSID");
        globusid = (globusid ? globusid : "GLOBUSID");
    }
    globusid_tmp = strdup(globusid);

    if (globus_buffer_desc.value)
    {
        gss_release_buffer(&minor_status2, globus_buffer);
    }
    return globusid_tmp;
}
